/*____________________________________________________________________
|
| File: tx.h	(Version 2.1s - August 1995)  shareware version
|
| Description: Interface to tx graphics library - LARGE model.  All
|	functions are far functions.
|
| Copyright (c) 1995 Timothy Edwin Roden
|___________________________________________________________________*/

/*___________________
|
| Constants
|__________________*/

#define txCLIP_OFF 0
#define txCLIP_ON  1

#define txFADE_IN 0

/* predefined colors */
#define txNO_COLOR 	    -1	/* used in text routines & get sprite functions */
#define txBLACK		    0
#define txBLUE		    1
#define	txGREEN		    2
#define txCYAN              3
#define txRED		    4
#define txMAGENTA     	    5
#define txBROWN             6
#define txGRAY		    7
#define txDARKGRAY	    8
#define txLIGHTBLUE         9
#define txLIGHTGREEN        10
#define txLIGHTCYAN         11
#define txLIGHTRED          12
#define txLIGHTMAGENTA      13
#define txYELLOW            14
#define txWHITE             15

/* font types */
#define txROM_FONT_8X8	0
#define txROM_FONT_8X14	1
#define txROM_FONT_8X16	2
#define txGEM_FONT	3

/* font spacing */
#define txFONT_SPACING_FIXED 	    0
#define txFONT_SPACING_PROPORTIONAL 1

/*___________________
|
| Type definitions
|__________________*/

typedef unsigned char byte;

typedef struct {
  int x,y;
} txPoint;

typedef struct {
  int x,y,w,h;
} txBound;

typedef struct {
  int x1,y1,x2,y2;
} txLine;

typedef struct {
  int x1,y1,x2,y2;
  int color;
} txLineDef;

typedef struct {
  int xleft, ytop, xright, ybottom;
} txRectangle;

typedef struct {
  txRectangle window;
  int	      window_active;
  txRectangle clip;
  int         clip_active;
  int         fore_color;
  int         active_page;
} txState;

typedef struct {
  int	    type;		/* rom or gem				*/
  int	    color;
  int	    backcolor;
  int	    spacing;		/* fixed or proportional		*/
  int	    inter_char_spacing;	/* pixel spacing between characters, 0-? */
  int       max_char_width;	/* pixel width of widest character 	*/
  int	    max_char_height;	/* pixel height of highest character	*/
  unsigned *cotptr;		/* character offset table		*/
  byte     *bufptr;		/* character buffer			*/
  unsigned  buf_width;		/* size of character buffer		*/
  unsigned  buf_height;
} txFont, *txFontPtr;

/*___________________
|
| Macros
|__________________*/

#define txTranslatePoint(pt,dx,dy)	\
  {					\
  pt.x = pt.x + dx;        		\
  pt.y = pt.y + dy;        		\
  }

#define txTranslateLine(line,dx,dy)	\
  {                                     \
  line.x1 = line.x1 + dx;           	\
  line.y1 = line.y1 + dy;           	\
  line.x2 = line.x2 + dx;           	\
  line.y2 = line.y2 + dy;           	\
  }

#define txTranslateRectangle(rect,dx,dy)	\
  {                                     	\
  rect.xleft   = rect.xleft   + dx;           	\
  rect.ytop    = rect.ytop    + dy;           	\
  rect.xright  = rect.xright  + dx;           	\
  rect.ybottom = rect.ybottom + dy;           	\
  }

#define txScalePoint(pt,xfac,yfac)	\
  {					\
  pt.x = pt.x * xfac;    		\
  pt.y = pt.y * yfac;    		\
  }

#define txScaleLine(line,xfac,yfac)	\
  {					\
  line.x1 = line.x1 * xfac;  	\
  line.y1 = line.y1 * yfac;  	\
  line.x2 = line.x2 * xfac;  	\
  line.y2 = line.y2 * yfac;  	\
  }

#define txScaleRectangle(rect,xfac,yfac)	\
  {                                     	\
  rect.xleft   = rect.xleft   * xfac;   	\
  rect.ytop    = rect.ytop    * yfac;   	\
  rect.xright  = rect.xright  * xfac;   	\
  rect.ybottom = rect.ybottom * yfac;   	\
  }

#define txSaveState(state)	\
  *(state) = tx_Current_state

#define txRestoreState(state)	\
  tx_Current_state = *(state)

#define txGetScreenWidth()	\
  (tx_Maxx+1)

#define txGetScreenHeight()	\
  (tx_Maxy+1)

#define txGetColor()	\
  tx_Current_state.fore_color

#define txGetWindow()	\
  tx_Current_state.window

#define txGetClip()	\
  tx_Current_state.clip

#define txSetClipping(clipping)	\
  tx_Current_state.clip_active = clipping

#define txGetClipping()	\
  tx_Current_state.clip_active

#define txGetMaxX()	\
  (tx_Current_state.window.xright - tx_Current_state.window.xleft)

#define txGetMaxY()	\
  (tx_Current_state.window.ybottom - tx_Current_state.window.ytop)

#define txGetActivePage()	\
  tx_Current_state.active_page

/*____________________________________________________________________
|
| Function prototypes
|___________________________________________________________________*/

/* file: TX.C */
void          far  txError (char far *str);	/* internal use only */
void 	      far  txOpenGraphics (int mode);
void 	      far  txCloseGraphics (void);
void 	      far  txSetVideoMode (int mode);
int 	      far  txGetVideoMode (void);
float	      far  txGetAspectRatio (void);
/* void	      far  txSaveState (txState far *save); */
/* void	      far  txRestoreState (txState far *save); */
/* int	      far  txGetScreenWidth (void); */
/* int	      far  txGetScreenHeight (void); */
void 	      far  txSetColor (int color);
/* int        far  txGetColor (void); */
void	      far  txSetWindow (txRectangle win);
/* txRectangle   far  txGetWindow (void); */
void	      far  txClearWindow (int color);
void	      far  txSetClip (txRectangle clip);
/* txRectangle   far  txGetClip (void); */
/* void	      far  txSetClipping (int clip); */
/* int	      far  txGetClipping (void); */
/* int	      far  txGetMaxX (void); */
/* int	      far  txGetMaxY (void); */
void 	      far  txSetActivePage (int page);
/* int 	      far  txGetActivePage (void); */
void 	      far  txSetVisualPage (int page);
int 	      far  txGetVisualPage (void);
void	      far  txClearPage (int page, int color);
unsigned long far  txImageSize  (txBound box);
unsigned long far  txSpriteSize (txBound box);
unsigned long far  txBitmapSize (txBound box);
void 	      far  txGetImage   (txBound box, byte far *image);
void 	      far  txGetSprite  (txBound box, byte far *sprite, int mask_color);
void	      far  txGetBitmap  (txBound box, byte far *bitmap);
byte 	      far *txCreateImage (txBound box);
byte 	      far *txCreateSprite (txBound box, int mask_color);
byte	      far *txCreateBitmap (txBound box);
void 	      far  txDrawImage  (byte far *image, int x, int y);
void 	      far  txDrawSprite (byte far *sprite, int x, int y);
void 	      far  txDrawSpriteMask (byte far *sprite, int x, int y, int color);
void	      far  txDrawBitmap (byte far *bitmap, int x, int y, int color);
void 	      far  txCopyPage (byte src_page, byte dest_page);
void	      far  txDrawPixel (int x, int y);
void	      far  txDrawLine (int x1, int y1, int x2, int y2);
void	      far  txDrawPoly (int num_points, txPoint *points);
void	      far  txDrawRectangle (int xleft, int ytop, int xright, int ybottom);
void	      far  txDrawFillRectangle (int xleft, int ytop, int xright, int ybottom);
void	      far  txDrawEllipse (int xctr, int yctr, int xradius, int yradius);
void	      far  txDrawCircle (int xctr, int yctr, int radius);

/* file: FILLPOLY.C */
void far txDrawFillPoly (int num_points, txPoint *points);

/* file: TXSCALE.C */
byte far *txScaleImage (byte far *image, float sx, float sy);
byte far *txScaleSprite (byte far *sprite, float sx, float sy);

/* file: TXEFFECT.C */
void far txDrawImageEffect (byte far *image, int x, int y, int effect);
void far txDrawSpriteEffect (byte far *sprite, int x, int y, int effect);
void far txDrawFillRectangleEffect (
  int xleft,
  int ytop,
  int xright,
  int ybottom,
  int effect );

byte      far  txReadImagePixel (byte far *image, int x, int y);
void      far  txWriteImagePixel (byte far *image, int x, int y, byte color);

/* file: TXCLIP.C */
int   	  far  txClipPoint (int x, int y);
int 	  far  txClipRectangle (int *xleft, int *ytop, int *xright, int *ybottom);
int 	  far  txClipLine (int *x1, int *y1, int *x2, int *y2);

/* file: TXTEXT.C */
txFontPtr far  txLoadFont (int font_type, char *filename);
void   	  far  txFreeFont (txFontPtr font);
void   	  far  txSetFont (txFontPtr font);
txFontPtr far  txGetFont (void);
void   	  far  txSetFontColor (int color);
int	  far  txGetFontColor (void);
void   	  far  txSetFontBackColor (int color);
int	  far  txGetFontBackColor (void);
void   	  far  txSetFontAttributes (int spacing, int inter_char_spacing);
void	  far  txGetFontAttributes (int *spacing, int *inter_char_spacing);
int 	  far  txGetFontWidth (void);
int 	  far  txGetFontHeight (void);
void	  far  txDrawText (char *str, int x, int y);
char far *txGetString (
  char far *str,	/* string to fill			*/
  int  max_str,		/* max characters to put into str, not counting NULL */
  int  x,		/* position in window to echo input	*/
  int  y,
  int  text_color,	/* color to echo input in		*/
  int  back_color, 	/* background color			*/
  void far (*animate_func)(void) );	/* ptr to callers animate function */

/* file: TXPIC.C */
int       far  txDrawLBMFile (char far *filename, byte page);

/* file: TXPCX.C */
int       far  txDrawPCXFile (char far *filename, byte page, int set_palette);
int       far  txGetPCXFile (char far *filename, byte page);

/* file: TXRANDOM.C */
void      far  txInitRandom (void);
int	  far  txGetRandom (int low, int high);
unsigned  far  txRandom_unsigned (void);
double    far  txRandom_double (void);

/* file: CLRBYTES.ASM */
void far txClearVGAMemory (
  unsigned int start_col,	/* 0-79 		*/
  unsigned int start_row,      	/* 0-199 		*/
  unsigned int cols,           	/* 1-80 		*/
  unsigned int rows,           	/* 1-200 		*/
  byte         page,          	/* 0-3  		*/
  byte         color );        	/* 0-15			*/

/* file: CPYBYTES.ASM */
void far txCopyVGAMemory (
  unsigned int src_col, 	/* 0-79 		*/
  unsigned int src_row,         /* 0-199 		*/
  byte	       src_pg,	 	/* 0-3			*/
  unsigned int dest_col,	/* 0-79 		*/
  unsigned int dest_row,        /* 0-199 		*/
  byte	       dest_pg,		/* 0-3			*/
  unsigned int cols,            /* 1-80 		*/
  unsigned int rows );          /* 1-200 		*/

/* file: VERTRET.ASM */
void far txVertRetraceDelay (void);

/* file: PALETTE.ASM */
void far txSetPalette (byte *rgb_palette);
void far txSetPaletteEntry (int entry, byte *rgb_value);
void far txGetPalette (byte *rgb_palette);
void far txGetPaletteEntry (int entry, byte *rgb_value);

/*___________________
|
| Global variables
|__________________*/

/* The following variables are used internally by TX and should not be
   changed by the user code. */

#ifdef _TX_
  #define GLOBAL /* */
#else
  #define GLOBAL extern
#endif

GLOBAL txState  tx_Current_state;
GLOBAL unsigned tx_Maxx;		/* max screen x coordinate */
GLOBAL unsigned tx_Maxy;		/* max screen y coordinate */

#undef GLOBAL
